/* ------------------------------------------------------------------------- */
/*
 *  test_convert.cpp
 *
 *  Copyright (c) 2004 - 2010, clown. All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *    - Redistributions of source code must retain the above copyright
 *      notice, this list of conditions and the following disclaimer.
 *    - Redistributions in binary form must reproduce the above copyright
 *      notice, this list of conditions and the following disclaimer in the
 *      documentation and/or other materials provided with the distribution.
 *    - No names of its contributors may be used to endorse or promote
 *      products derived from this software without specific prior written
 *      permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *  Last-modified: Mon 01 Mar 2010 20:12:00 JST
 */
/* ------------------------------------------------------------------------- */
#include <cppunit/extensions/HelperMacros.h>
#include <exception>
#include <string>
#include <sstream>
#include "clx/convert.h"
#include "clx/convert_policy.h"

/* ------------------------------------------------------------------------- */
//  test_convert
/* ------------------------------------------------------------------------- */
class test_convert : public CPPUNIT_NS::TestFixture {
public:
	CPPUNIT_TEST_SUITE(test_convert);
	CPPUNIT_TEST(test_dec);
	CPPUNIT_TEST(test_hex);
	CPPUNIT_TEST(test_uri);
	CPPUNIT_TEST(test_html);
	CPPUNIT_TEST(test_base64);
	CPPUNIT_TEST_SUITE_END();
	
public:
	test_convert() : CPPUNIT_NS::TestFixture() {}
	virtual ~test_convert() throw() {}
	void setUp() {}
	void tearDown() {}
	
protected:
	void test_dec();
	void test_hex();
	void test_uri();
	void test_html();
	void test_base64();
};

/* ------------------------------------------------------------------------- */
//  registration
/* ------------------------------------------------------------------------- */
CPPUNIT_TEST_SUITE_REGISTRATION(test_convert);

/* ------------------------------------------------------------------------- */
//  test_convert::test_dec
/* ------------------------------------------------------------------------- */
void test_convert::test_dec() {
	clx::dec_encoder f("&#", ";");
	std::string src;
	std::string valid;
	std::string encoded;
	
	src = "a";
	valid = "&#97;";
	encoded = clx::convert(src, f);
	CPPUNIT_ASSERT_MESSAGE(encoded, encoded == valid);
	
	src = "Hello, world!";
	valid = "&#72;&#101;&#108;&#108;&#111;&#44;&#32;&#119;&#111;&#114;&#108;&#100;&#33;";
	encoded = clx::convert(src, f);
	CPPUNIT_ASSERT_MESSAGE(encoded, encoded == valid);
	
	src.clear();
	encoded = clx::convert(src, f);
	CPPUNIT_ASSERT(encoded.empty());
}

/* ------------------------------------------------------------------------- */
//  test_convert::test_hex
/* ------------------------------------------------------------------------- */
void test_convert::test_hex() {
	clx::hex_encoder f("&#x", ";");
	std::string src;
	std::string valid;
	std::string encoded;
	
	src = "a";
	valid = "&#x61;";
	encoded = clx::convert(src, f);
	CPPUNIT_ASSERT_MESSAGE(encoded, encoded == valid);
	
	src = "Hello, world!";
	valid = "&#x48;&#x65;&#x6c;&#x6c;&#x6f;&#x2c;&#x20;&#x77;&#x6f;&#x72;&#x6c;&#x64;&#x21;";
	encoded = clx::convert(src, f);
	CPPUNIT_ASSERT_MESSAGE(encoded, encoded == valid);
	
	src.clear();
	encoded = clx::convert(src, f);
	CPPUNIT_ASSERT(encoded.empty());
}

/* ------------------------------------------------------------------------- */
//  test_convert::test_uri
/* ------------------------------------------------------------------------- */
void test_convert::test_uri() {
	/*
	 * encode symbols except for ".", convert space (0x20) to plus,
	 * and put hex numbers with upper case (lower = false).
	 */
	clx::uri_encoder encoder(".", true, false);
	clx::uri_decoder decoder;
	std::string src;
	std::string valid;
	std::string encoded;
	std::string decoded;
	
	src = "a";
	valid = "a";
	encoded = clx::convert(src, encoder);
	CPPUNIT_ASSERT_MESSAGE(encoded, encoded == valid);
	decoded = clx::convert(encoded, decoder);
	CPPUNIT_ASSERT_MESSAGE(decoded, decoded == src);
	
	src = "いろはにほへと";
	valid = "%E3%81%84%E3%82%8D%E3%81%AF%E3%81%AB%E3%81%BB%E3%81%B8%E3%81%A8";
	encoded = clx::convert(src, encoder);
	CPPUNIT_ASSERT_MESSAGE(encoded, encoded == valid);
	decoded = clx::convert(encoded, decoder);
	CPPUNIT_ASSERT_MESSAGE(decoded, decoded == src);
	
	src = "Hello, world!いろはにほへと";
	valid = "Hello%2C+world%21%E3%81%84%E3%82%8D%E3%81%AF%E3%81%AB%E3%81%BB%E3%81%B8%E3%81%A8";
	encoded = clx::convert(src, encoder);
	CPPUNIT_ASSERT_MESSAGE(encoded, encoded == valid);
	decoded = clx::convert(encoded, decoder);
	CPPUNIT_ASSERT_MESSAGE(decoded, decoded == src);
	
	src = "http://www.example.com/";
	valid = "http%3A%2F%2Fwww.example.com%2F";
	encoded = clx::convert(src, encoder);
	CPPUNIT_ASSERT_MESSAGE(encoded, encoded == valid);
	decoded = clx::convert(encoded, decoder);
	CPPUNIT_ASSERT_MESSAGE(decoded, decoded == src);
	
	src = "http://www.example.com/~foo/bar.cgi?hoge=fuga&boke=moka";
	valid = "http%3A%2F%2Fwww.example.com%2F%7Efoo%2Fbar.cgi%3Fhoge%3Dfuga%26boke%3Dmoka";
	encoded = clx::convert(src, encoder);
	CPPUNIT_ASSERT_MESSAGE(encoded, encoded == valid);
	decoded = clx::convert(encoded, decoder);
	CPPUNIT_ASSERT_MESSAGE(decoded, decoded == src);
	
	src.clear();
	encoded = clx::convert(src, encoder);
	CPPUNIT_ASSERT(encoded.empty());
	decoded = clx::convert(src, decoder);
	CPPUNIT_ASSERT(decoded.empty());
}

/* ------------------------------------------------------------------------- */
//  test_convert::test_html
/* ------------------------------------------------------------------------- */
void test_convert::test_html() {
	clx::html_encoder encoder;
	clx::html_decoder decoder;
	std::string src;
	std::string valid;
	std::string encoded;
	std::string decoded;
	
	src = "a";
	valid = "a";
	encoded = clx::convert(src, encoder);
	CPPUNIT_ASSERT_MESSAGE(encoded, encoded == valid);
	decoded = clx::convert(encoded, decoder);
	CPPUNIT_ASSERT_MESSAGE(decoded, decoded == src);
	
	src = "Hello, world!";
	valid = "Hello, world!";
	encoded = clx::convert(src, encoder);
	CPPUNIT_ASSERT_MESSAGE(encoded, encoded == valid);
	decoded = clx::convert(encoded, decoder);
	CPPUNIT_ASSERT_MESSAGE(decoded, decoded == src);
	
	src = "いろはにほへと";
	valid = "いろはにほへと";
	encoded = clx::convert(src, encoder);
	CPPUNIT_ASSERT_MESSAGE(encoded, encoded == valid);
	decoded = clx::convert(encoded, decoder);
	CPPUNIT_ASSERT_MESSAGE(decoded, decoded == src);
	
	src = "<img src=\"hoge.jpg\", alt=\"hoge\">";
	valid = "&lt;img src=&quot;hoge.jpg&quot;, alt=&quot;hoge&quot;&gt;";
	encoded = clx::convert(src, encoder);
	CPPUNIT_ASSERT_MESSAGE(encoded, encoded == valid);
	decoded = clx::convert(encoded, decoder);
	CPPUNIT_ASSERT_MESSAGE(decoded, decoded == src);
	
	src = "foo<>&\"'いろはにほへと";
	valid = "foo&lt;&gt;&amp;&quot;&#39;いろはにほへと";
	encoded = clx::convert(src, encoder);
	CPPUNIT_ASSERT_MESSAGE(encoded, encoded == valid);
	decoded = clx::convert(encoded, decoder);
	CPPUNIT_ASSERT_MESSAGE(decoded, decoded == src);
	
	src.clear();
	encoded = clx::convert(src, encoder);
	CPPUNIT_ASSERT(encoded.empty());
	decoded = clx::convert(src, decoder);
	CPPUNIT_ASSERT(decoded.empty());
}

/* ------------------------------------------------------------------------- */
//  test_convert::test_base64
/* ------------------------------------------------------------------------- */
void test_convert::test_base64() {
	clx::base64_encoder encoder;
	clx::base64_decoder decoder;
	std::string src;
	std::string valid;
	std::string encoded;
	std::string decoded;
	
	src = "a";
	valid = "YQ==";
	encoded = clx::convert(src, encoder);
	CPPUNIT_ASSERT_MESSAGE(encoded, encoded == valid);
	decoded = clx::convert(encoded, decoder);
	CPPUNIT_ASSERT_MESSAGE(decoded, decoded == src);
	
	src = "ab";
	valid = "YWI=";
	encoded = clx::convert(src, encoder);
	CPPUNIT_ASSERT_MESSAGE(encoded, encoded == valid);
	decoded = clx::convert(encoded, decoder);
	CPPUNIT_ASSERT_MESSAGE(decoded, decoded == src);
	
	src = "abc";
	valid = "YWJj";
	encoded = clx::convert(src, encoder);
	CPPUNIT_ASSERT_MESSAGE(encoded, encoded == valid);
	decoded = clx::convert(encoded, decoder);
	CPPUNIT_ASSERT_MESSAGE(decoded, decoded == src);
	
	src = "abcd";
	valid = "YWJjZA==";
	encoded = clx::convert(src, encoder);
	CPPUNIT_ASSERT_MESSAGE(encoded, encoded == valid);
	decoded = clx::convert(encoded, decoder);
	CPPUNIT_ASSERT_MESSAGE(decoded, decoded == src);
	
	src = "Hello, world!";
	valid = "SGVsbG8sIHdvcmxkIQ==";
	encoded = clx::convert(src, encoder);
	CPPUNIT_ASSERT_MESSAGE(encoded, encoded == valid);
	decoded = clx::convert(encoded, decoder);
	CPPUNIT_ASSERT_MESSAGE(decoded, decoded == src);
	
	src = "いろはにほへと";
	valid = "44GE44KN44Gv44Gr44G744G444Go";
	encoded = clx::convert(src, encoder);
	CPPUNIT_ASSERT_MESSAGE(encoded, encoded == valid);
	decoded = clx::convert(encoded, decoder);
	CPPUNIT_ASSERT_MESSAGE(decoded, decoded == src);
	
	src = "Hello, world! いろはにほへと";
	valid = "SGVsbG8sIHdvcmxkISDjgYTjgo3jga/jgavjgbvjgbjjgag=";
	encoded = clx::convert(src, encoder);
	CPPUNIT_ASSERT_MESSAGE(encoded, encoded == valid);
	decoded = clx::convert(encoded, decoder);
	CPPUNIT_ASSERT_MESSAGE(decoded, decoded == src);
	
	src.clear();
	encoded = clx::convert(src, encoder);
	CPPUNIT_ASSERT(encoded.empty());
	decoded = clx::convert(src, decoder);
	CPPUNIT_ASSERT(decoded.empty());
}
