/* ------------------------------------------------------------------------- */
/*
 *  test_filesystem.cpp
 *
 *  Copyright (c) 2004 - 2010, clown. All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *    - Redistributions of source code must retain the above copyright
 *      notice, this list of conditions and the following disclaimer.
 *    - Redistributions in binary form must reproduce the above copyright
 *      notice, this list of conditions and the following disclaimer in the
 *      documentation and/or other materials provided with the distribution.
 *    - No names of its contributors may be used to endorse or promote
 *      products derived from this software without specific prior written
 *      permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *  Last-modified: Wed 20 Jan 2010 13:39:00 JST
 */
/* ------------------------------------------------------------------------- */
#include <cppunit/extensions/HelperMacros.h>
#include <exception>
#include <stdexcept>
#include <string>
#include <vector>
#include <iterator>
#include "clx/filesystem.h"

/* ------------------------------------------------------------------------- */
//  test_filesystem
/* ------------------------------------------------------------------------- */
class test_filesystem : public CPPUNIT_NS::TestFixture {
	CPPUNIT_TEST_SUITE(test_filesystem);
	CPPUNIT_TEST(test_extension);
	CPPUNIT_TEST(test_basename);
	CPPUNIT_TEST(test_leaf_and_branch);
	CPPUNIT_TEST(test_exists);
	CPPUNIT_TEST(test_copy);
	CPPUNIT_TEST(test_list);
	CPPUNIT_TEST_SUITE_END();
	
public:
	test_filesystem() : CPPUNIT_NS::TestFixture() {}
	virtual ~test_filesystem() throw() {}
	void setUp() {}
	void tearDown() {}
	
protected:
	void test_extension();
	void test_basename();
	void test_leaf_and_branch();
	void test_exists();
	void test_copy();
	void test_list();
};

/* ------------------------------------------------------------------------- */
//  registration
/* ------------------------------------------------------------------------- */
CPPUNIT_TEST_SUITE_REGISTRATION(test_filesystem);

/* ------------------------------------------------------------------------- */
//  test_filesystem::test_extension
/* ------------------------------------------------------------------------- */
void test_filesystem::test_extension() {
	std::string s1("sample.txt");
	CPPUNIT_ASSERT(clx::filesystem::extension(s1) == ".txt");
	
	std::string s2("sample");
	CPPUNIT_ASSERT(clx::filesystem::extension(s2) == s2);
	
	std::string s3("sample.tar.gz");
	CPPUNIT_ASSERT(clx::filesystem::extension(s3) == ".gz");
	
	std::string s4("foo/bar/sample.txt");
	CPPUNIT_ASSERT(clx::filesystem::extension(s4) == ".txt");
	
	std::string s5("foo/bar/bas");
	CPPUNIT_ASSERT(clx::filesystem::extension(s5) == s5);
	
	std::string s6("");
	CPPUNIT_ASSERT(clx::filesystem::extension(s6).empty());
}

/* ------------------------------------------------------------------------- */
//  test_filesystem::test_leaf_and_branch
/* ------------------------------------------------------------------------- */
void test_filesystem::test_leaf_and_branch() {
	std::string s1("foo.bar");
	CPPUNIT_ASSERT(clx::filesystem::leaf(s1) == "foo.bar");
	CPPUNIT_ASSERT(clx::filesystem::branch(s1).empty());
	
	std::string s2("foo/bar.bas");
	CPPUNIT_ASSERT(clx::filesystem::leaf(s2) == "bar.bas");
	CPPUNIT_ASSERT(clx::filesystem::branch(s2) == "foo");
	
	std::string s3("foo/bar/");
	CPPUNIT_ASSERT(clx::filesystem::leaf(s3).empty());
	CPPUNIT_ASSERT(clx::filesystem::branch(s3) == "foo/bar");
	
	std::string s4("/foo/bar/bas.hoge");
	CPPUNIT_ASSERT(clx::filesystem::leaf(s4) == "bas.hoge");
	CPPUNIT_ASSERT(clx::filesystem::branch(s4) == "/foo/bar");
	
	std::string s5("foo\\bar.bas");
	CPPUNIT_ASSERT(clx::filesystem::leaf(s5) == "bar.bas");
	CPPUNIT_ASSERT(clx::filesystem::branch(s5) == "foo");
	
	std::string s6("C:\\foo\\bar\\bas.hoge");
	CPPUNIT_ASSERT(clx::filesystem::leaf(s6) == "bas.hoge");
	CPPUNIT_ASSERT(clx::filesystem::branch(s6) == "C:\\foo\\bar");
	
	std::string s7("C:\\foo\\");
	CPPUNIT_ASSERT(clx::filesystem::leaf(s7).empty());
	CPPUNIT_ASSERT(clx::filesystem::branch(s7) == "C:\\foo");
	
	std::string s8("");
	CPPUNIT_ASSERT(clx::filesystem::leaf(s8).empty());
	CPPUNIT_ASSERT(clx::filesystem::branch(s8).empty());
}

/* ------------------------------------------------------------------------- */
//  test_filesystem::test_basename
/* ------------------------------------------------------------------------- */
void test_filesystem::test_basename() {
	std::string s1("sample.txt");
	CPPUNIT_ASSERT(clx::filesystem::basename(s1) == "sample");
	
	std::string s2("sample");
	CPPUNIT_ASSERT(clx::filesystem::basename(s2) == s2);
	
	std::string s3("sample.tar.gz");
	CPPUNIT_ASSERT(clx::filesystem::basename(s3) == "sample.tar");
	
	std::string s4("foo/bar/sample.txt");
	CPPUNIT_ASSERT(clx::filesystem::basename(s4) == "foo/bar/sample");
	
	std::string s5("foo/bar/bas");
	CPPUNIT_ASSERT(clx::filesystem::basename(s5) == s5);
	
	std::string s6("");
	CPPUNIT_ASSERT(clx::filesystem::basename(s6).empty());
}

/* ------------------------------------------------------------------------- */
//  test_filesystem::test_exists
/* ------------------------------------------------------------------------- */
void test_filesystem::test_exists() {
	std::string s1("test_filesystem.cpp");
	CPPUNIT_ASSERT(clx::filesystem::exists(s1));
	CPPUNIT_ASSERT(!clx::filesystem::is_directory(s1));
	
	std::string s2("test_filesystem.cpp.bak");
	CPPUNIT_ASSERT(!clx::filesystem::exists(s2));
	CPPUNIT_ASSERT(!clx::filesystem::is_directory(s2));
	
	std::string s3("samples/");
	CPPUNIT_ASSERT(clx::filesystem::exists(s3));
	CPPUNIT_ASSERT(clx::filesystem::is_directory(s3));
	
	std::string s4("samples");
	CPPUNIT_ASSERT(clx::filesystem::exists(s4));
	CPPUNIT_ASSERT(clx::filesystem::is_directory(s4));
	
	std::string s5("dummy");
	CPPUNIT_ASSERT(!clx::filesystem::exists(s5));
	CPPUNIT_ASSERT(!clx::filesystem::is_directory(s5));
}

/* ------------------------------------------------------------------------- */
//  test_filesystem::test_copy
/* ------------------------------------------------------------------------- */
void test_filesystem::test_copy() {
	std::string from1("test_filesystem.cpp");
	std::string to1("test_filesystem.txt");
	try {
		if (!clx::filesystem::copy(from1, to1)) throw std::runtime_error("1st try.");
		if (!clx::filesystem::copy(from1, to1)) throw std::runtime_error("2nd try (allow overwrite).");
		if (clx::filesystem::copy(from1, to1, true)) throw std::runtime_error("3rd try (deny overwrite).");
		CPPUNIT_ASSERT(clx::filesystem::exists(to1));
		std::remove(to1.c_str());
		CPPUNIT_ASSERT(!clx::filesystem::exists(to1));
	}
	catch (std::runtime_error& e) {
		if (clx::filesystem::exists(to1)) std::remove(to1.c_str());
		CPPUNIT_FAIL(e.what());
	}
	
	std::string dummy("dummy.txt");
	try {
		std::string from2("test_filesystem_err.cpp");
		CPPUNIT_ASSERT(!clx::filesystem::copy(from2, dummy));
		CPPUNIT_ASSERT(!clx::filesystem::exists(dummy));
		
		std::string from3("samples/");
		CPPUNIT_ASSERT(!clx::filesystem::copy(from3, dummy));
		CPPUNIT_ASSERT(!clx::filesystem::exists(dummy));
	}
	catch (std::exception& e) {
		if (clx::filesystem::exists(dummy)) std::remove(dummy.c_str());
		CPPUNIT_FAIL(e.what());
	}
}

/* ------------------------------------------------------------------------- */
//  test_filesystem::test_list
/* ------------------------------------------------------------------------- */
void test_filesystem::test_list() {
	std::string dir1("samples");
	std::vector<std::string> v1;
	clx::filesystem::list(dir1, std::back_inserter(v1));
	CPPUNIT_ASSERT(!v1.empty());
	
	std::string dir2("samples/");
	std::vector<std::string> v2;
	clx::filesystem::list(dir2, std::back_inserter(v2));
	CPPUNIT_ASSERT(!v2.empty());
	
	std::string dir3(".");
	std::vector<std::string> v3;
	clx::filesystem::list(dir3, std::back_inserter(v3));
	CPPUNIT_ASSERT(!v3.empty());
	
	std::string dir4("..");
	std::vector<std::string> v4;
	clx::filesystem::list(dir4, std::back_inserter(v4));
	CPPUNIT_ASSERT(!v4.empty());
	
	std::string dir5("../test");
	std::vector<std::string> v5;
	clx::filesystem::list(dir5, std::back_inserter(v5));
	CPPUNIT_ASSERT(!v5.empty());
	
	std::string dir6("dummy");
	std::vector<std::string> v6;
	clx::filesystem::list(dir6, std::back_inserter(v6));
	CPPUNIT_ASSERT(v6.empty());
	
	std::string dir7("test_filesystem.cpp");
	std::vector<std::string> v7;
	clx::filesystem::list(dir7, std::back_inserter(v7));
	CPPUNIT_ASSERT(v7.empty());
}
