/* ------------------------------------------------------------------------- */
/*
 *  test_http.h
 *
 *  Copyright (c) 2004 - 2010, clown. All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *    - Redistributions of source code must retain the above copyright
 *      notice, this list of conditions and the following disclaimer.
 *    - Redistributions in binary form must reproduce the above copyright
 *      notice, this list of conditions and the following disclaimer in the
 *      documentation and/or other materials provided with the distribution.
 *    - No names of its contributors may be used to endorse or promote
 *      products derived from this software without specific prior written
 *      permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *  Last-modified: Thu 17 Jun 2010 22:01:00 JST
 */
/* ------------------------------------------------------------------------- */
#include <cppunit/extensions/HelperMacros.h>
#include <exception>
#include <fstream>
#include <iterator>
#include "clx/json.h"
#include "clx/lexical_cast.h"

/* ------------------------------------------------------------------------- */
//  test_json
/* ------------------------------------------------------------------------- */
class test_json : public CPPUNIT_NS::TestFixture {
	CPPUNIT_TEST_SUITE(test_json);
	CPPUNIT_TEST(test_object);
	CPPUNIT_TEST(test_object_invalid);
	CPPUNIT_TEST(test_array);
	CPPUNIT_TEST(test_array_invalid);
	CPPUNIT_TEST(test_complex_data);
	CPPUNIT_TEST_SUITE_END();
	
public:
	test_json() : CPPUNIT_NS::TestFixture() {}
	virtual ~test_json() throw() {}
	void setUp() {}
	void tearDown() {}
	
protected:
	void test_object();
	void test_object_invalid();
	void test_array();
	void test_array_invalid();
	void test_complex_data();
};

/* ------------------------------------------------------------------------- */
//  registration
/* ------------------------------------------------------------------------- */
CPPUNIT_TEST_SUITE_REGISTRATION(test_json);

/* ------------------------------------------------------------------------- */
//  test_http::test_object
/* ------------------------------------------------------------------------- */
void test_json::test_object() {
	try {
		const std::string s1 = "{}";
		clx::json j1(s1);
		CPPUNIT_ASSERT(j1.empty());
		
		const std::string s2 = "{\"foo\":\"bar\",\"hoge\":\"fuga\"}";
		clx::json j2(s2);
		CPPUNIT_ASSERT(j2.size() == 2);
		
		const std::string s3 = "{\"foo\":\"bar\",\"count\":1,\"boolean\":true}";
		clx::json j3(s3);
		CPPUNIT_ASSERT(j3.size() == 3);
	}
	catch (std::exception& e) {
		CPPUNIT_FAIL(e.what());
	}
}

/* ------------------------------------------------------------------------- */
//  test_http::test_object_invalid
/* ------------------------------------------------------------------------- */
void test_json::test_object_invalid() {
	try {
		const std::string s1 = "\"foo";
		clx::json j1(s1);
		CPPUNIT_ASSERT_MESSAGE(clx::lexical_cast<std::string>(j1.size()), j1.empty());
		
		/*
		 * Case where '{' and/or '}' characters are missing.
		 * The json class ignores first '{' and last '}' characters,
		 * so the folloing string are passed.
		 */
		const std::string s2 = "\"foo\":\"bar\",\"count\":1,\"boolean\":true}";
		clx::json j2(s2);
		CPPUNIT_ASSERT_MESSAGE(clx::lexical_cast<std::string>(j2.size()), j2.size() == 3);
		
		const std::string s3 = "{\"foo\":\"bar\",\"count\":1,\"boolean\":true";
		clx::json j3(s3);
		CPPUNIT_ASSERT_MESSAGE(clx::lexical_cast<std::string>(j3.size()), j3.size() == 3);
		
		const std::string s4 = "\"foo\":\"bar\",\"count\":1,\"boolean\":true";
		clx::json j4(s4);
		CPPUNIT_ASSERT_MESSAGE(clx::lexical_cast<std::string>(j4.size()), j4.size() == 3);
		
		// JSON object allows only pair element.
		const std::string s5 = "{\"foo\",\"bar\",\"count\":1,\"boolean\":true}";
		clx::json j5(s5);
		CPPUNIT_ASSERT_MESSAGE(clx::lexical_cast<std::string>(j5.size()), j5.empty());
		
		// clx::json stops parsing until finds an error, so the first element is passed.
		const std::string s6 = "\"foo\":\"bar\",\"count\",1,\"boolean\":true";
		clx::json j6(s6);
		CPPUNIT_ASSERT_MESSAGE(clx::lexical_cast<std::string>(j6.size()), j6.size() == 1);
		
		const std::string s7 = "{1:\"bar\",\"count\":1,\"boolean\":true}";
		clx::json j7(s7);
		CPPUNIT_ASSERT_MESSAGE(clx::lexical_cast<std::string>(j7.size()), j7.empty());
		
		const std::string s8 = "{\"hello\":\"world\",\"results\":[{\"foo\":\"bar\"}";
		clx::json j8(s8);
		CPPUNIT_ASSERT_MESSAGE(clx::lexical_cast<std::string>(j8.size()), j8.size() == 1);
	}
	catch (std::exception& e) {
		CPPUNIT_FAIL(e.what());
	}
}

/* ------------------------------------------------------------------------- */
//  test_http::test_array
/* ------------------------------------------------------------------------- */
void test_json::test_array() {
	try {
		const std::string s1 = "{\"results\":[]}";
		clx::json j1(s1);
		CPPUNIT_ASSERT(j1.size() == 1);
		CPPUNIT_ASSERT(j1.find("results") != j1.end());
		clx::json_array a1(j1["results"]);
		CPPUNIT_ASSERT_MESSAGE(clx::lexical_cast<std::string>(a1.objects().size()), a1.objects().empty());
		CPPUNIT_ASSERT_MESSAGE(clx::lexical_cast<std::string>(a1.strings().size()), a1.strings().empty());
		
		const std::string s2 = "{\"results\":[{\"foo\":\"bar\"}]}";
		clx::json j2(s2);
		CPPUNIT_ASSERT(j2.size() == 1);
		CPPUNIT_ASSERT(j2.find("results") != j2.end());
		clx::json_array a2(j2["results"]);
		CPPUNIT_ASSERT_MESSAGE(clx::lexical_cast<std::string>(a2.objects().size()), a2.objects().size() == 1);
		CPPUNIT_ASSERT_MESSAGE(clx::lexical_cast<std::string>(a2.strings().size()), a2.strings().empty());
		
		const std::string s3 = "{\"results\":[{\"foo\":\"bar\",\"count\":1,\"boolean\":true}]}";
		clx::json j3(s3);
		CPPUNIT_ASSERT(j3.size() == 1);
		CPPUNIT_ASSERT(j3.find("results") != j3.end());
		clx::json_array a3(j3["results"]);
		CPPUNIT_ASSERT_MESSAGE(clx::lexical_cast<std::string>(a3.objects().size()), a3.objects().size() == 1);
		CPPUNIT_ASSERT_MESSAGE(clx::lexical_cast<std::string>(a3.strings().size()), a3.strings().empty());
		CPPUNIT_ASSERT_MESSAGE(clx::lexical_cast<std::string>(a3.object(0).size()), a3.object(0).size() == 3);
		
		const std::string s4 = "{\"results\":[\"foo\",\"bar\",\"count\",1,\"boolean\",true]}";
		clx::json j4(s4);
		CPPUNIT_ASSERT(j4.size() == 1);
		CPPUNIT_ASSERT(j4.find("results") != j4.end());
		clx::json_array a4(j4["results"]);
		CPPUNIT_ASSERT_MESSAGE(clx::lexical_cast<std::string>(a4.objects().size()), a4.objects().empty());
		CPPUNIT_ASSERT_MESSAGE(clx::lexical_cast<std::string>(a4.strings().size()), a4.strings().size() == 6);
		
		const std::string s5 = "{\"results\":[{\"foo\":\"bar\",\"count\":1,\"boolean\":true},\"baz\",25600,false]}";
		clx::json j5(s5);
		CPPUNIT_ASSERT(j5.size() == 1);
		CPPUNIT_ASSERT(j5.find("results") != j5.end());
		clx::json_array a5(j5["results"]);
		CPPUNIT_ASSERT_MESSAGE(clx::lexical_cast<std::string>(a5.objects().size()), a5.objects().size() == 1);
		CPPUNIT_ASSERT_MESSAGE(clx::lexical_cast<std::string>(a5.strings().size()), a5.strings().size() == 3);
		CPPUNIT_ASSERT_MESSAGE(clx::lexical_cast<std::string>(a5.object(0).size()), a5.object(0).size() == 3);
		
		const std::string s6 = "{\"results\":[{\"foo\":\"bar\",\"count\":1,\"boolean\":true},\"baz\",25600,false,[\"nested\":\"hello\"]]}";
		clx::json j6(s6);
		CPPUNIT_ASSERT(j6.size() == 1);
		CPPUNIT_ASSERT(j6.find("results") != j6.end());
		clx::json_array a6(j6["results"]);
		CPPUNIT_ASSERT_MESSAGE(clx::lexical_cast<std::string>(a6.objects().size()), a6.objects().size() == 1);
		CPPUNIT_ASSERT_MESSAGE(clx::lexical_cast<std::string>(a6.strings().size()), a6.strings().size() == 4);
		CPPUNIT_ASSERT_MESSAGE(clx::lexical_cast<std::string>(a6.object(0).size()), a6.object(0).size() == 3);
		
		const std::string s7 = "{\"hello\":\"world\",\"results\":[{\"foo\":\"bar\",\"count\":1,\"boolean\":true},\"baz\",25600,false,]}";
		clx::json j7(s7);
		CPPUNIT_ASSERT(j7.size() == 2);
		CPPUNIT_ASSERT(j7.find("results") != j7.end());
		clx::json_array a7(j7["results"]);
		CPPUNIT_ASSERT_MESSAGE(clx::lexical_cast<std::string>(a7.objects().size()), a7.objects().size() == 1);
		CPPUNIT_ASSERT_MESSAGE(clx::lexical_cast<std::string>(a7.strings().size()), a7.strings().size() == 3);
		CPPUNIT_ASSERT_MESSAGE(clx::lexical_cast<std::string>(a7.object(0).size()), a7.object(0).size() == 3);
	}
	catch (std::exception& e) {
		CPPUNIT_FAIL(e.what());
	}
}

/* ------------------------------------------------------------------------- */
//  test_http::test_array_invalid
/* ------------------------------------------------------------------------- */
void test_json::test_array_invalid() {
	try {
		const std::string s1 = "{\"results\":[\"foo]}";
		clx::json j1(s1);
		CPPUNIT_ASSERT(j1.size() == 1);
		CPPUNIT_ASSERT(j1.find("results") != j1.end());
		clx::json_array a1(j1["results"]);
		CPPUNIT_ASSERT_MESSAGE(clx::lexical_cast<std::string>(a1.objects().size()), a1.objects().empty());
		CPPUNIT_ASSERT_MESSAGE(clx::lexical_cast<std::string>(a1.strings().size()), a1.strings().empty());
		
		// element in the object is incomplete.
		const std::string s2 = "{\"results\":[{\"foo\"}]}";
		clx::json j2(s2);
		CPPUNIT_ASSERT(j2.size() == 1);
		CPPUNIT_ASSERT(j2.find("results") != j2.end());
		clx::json_array a2(j2["results"]);
		CPPUNIT_ASSERT_MESSAGE(clx::lexical_cast<std::string>(a2.objects().size()), a2.objects().empty());
		CPPUNIT_ASSERT_MESSAGE(clx::lexical_cast<std::string>(a2.strings().size()), a2.strings().empty());
		
		// "foo" does not end '"' character.
		const std::string s3 = "{\"results\":[{\"foo:\"bar\",\"count\":1,\"boolean\":true}]}";
		clx::json j3(s3);
		CPPUNIT_ASSERT(j3.size() == 1);
		CPPUNIT_ASSERT(j3.find("results") != j3.end());
		clx::json_array a3(j3["results"]);
		CPPUNIT_ASSERT_MESSAGE(clx::lexical_cast<std::string>(a3.objects().size()), a3.objects().empty());
		CPPUNIT_ASSERT_MESSAGE(clx::lexical_cast<std::string>(a3.strings().size()), a3.strings().empty());
		
		// '}' character is missing.
		const std::string s4 = "{\"results\":[{\"foo\":\"bar\",\"count\":1,\"boolean\":true]}";
		clx::json j4(s4);
		CPPUNIT_ASSERT(j4.size() == 1);
		CPPUNIT_ASSERT(j4.find("results") != j4.end());
		clx::json_array a4(j4["results"]);
		CPPUNIT_ASSERT_MESSAGE(clx::lexical_cast<std::string>(a4.objects().size()), a4.objects().empty());
		CPPUNIT_ASSERT_MESSAGE(clx::lexical_cast<std::string>(a4.strings().size()), a4.strings().empty());
		
		// ']' character is missing.
		// In these case, clx::json finds the error before clx::json_array starts parsing.
		const std::string s5 = "{\"results\":[{\"foo\":\"bar\",\"count\":1,\"boolean\":true},\"baz\",25600,false,[\"nested\":\"hello\"}]";
		clx::json j5(s5);
		CPPUNIT_ASSERT(j5.empty());
		
		const std::string s6 = "{\"results\":[{\"foo\":\"bar\",\"count\":1,\"boolean\":true},\"baz\",25600,false,[\"nested\":\"hello\"}";
		clx::json j6(s6);
		CPPUNIT_ASSERT(j6.empty());
	}
	catch (std::exception& e) {
		CPPUNIT_FAIL(e.what());
	}
}

/* ------------------------------------------------------------------------- */
//  test_http::test_complex_data
/* ------------------------------------------------------------------------- */
void test_json::test_complex_data() {
	try {
		std::ifstream ifs("samples/sample_json.txt");
		std::istreambuf_iterator<char> input(ifs);
		std::istreambuf_iterator<char> last;
		
		std::string s1;
		std::insert_iterator<std::string> output(s1, s1.end());
		
		std::copy(input, last, output);
		
		clx::json j1(s1);
		CPPUNIT_ASSERT(j1.size() == 9);
		CPPUNIT_ASSERT(j1.find("results") != j1.end());
		
		clx::json_array j2(j1["results"]);
		CPPUNIT_ASSERT(j2.objects().size() == 2);
		CPPUNIT_ASSERT(j2.strings().empty());
		
		for (std::size_t i = 0; i < j2.objects().size(); ++i) {
			CPPUNIT_ASSERT_MESSAGE(clx::lexical_cast<std::string>(j2.object(i).size()), j2.object(i).size() == 11);
		}
	}
	catch (std::exception& e) {
		CPPUNIT_FAIL(e.what());
	}
}
