/* ------------------------------------------------------------------------- */
/*
 *  test_sha1.cpp
 *
 *  Copyright (c) 2004 - 2010, clown. All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *    - Redistributions of source code must retain the above copyright
 *      notice, this list of conditions and the following disclaimer.
 *    - Redistributions in binary form must reproduce the above copyright
 *      notice, this list of conditions and the following disclaimer in the
 *      documentation and/or other materials provided with the distribution.
 *    - No names of its contributors may be used to endorse or promote
 *      products derived from this software without specific prior written
 *      permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *  Last-modified: Tue 12 Jan 2010 02:32:00 JST
 */
/* ------------------------------------------------------------------------- */
#include <cppunit/extensions/HelperMacros.h>
#include <exception>
#include <string>
#include "clx/sha1.h"
#include "clx/hmac.h"

/* ------------------------------------------------------------------------- */
//  test_sha1
/* ------------------------------------------------------------------------- */
class test_sha1 : public CPPUNIT_NS::TestFixture {
	CPPUNIT_TEST_SUITE(test_sha1);
	CPPUNIT_TEST(test_normal);
	CPPUNIT_TEST(test_hmac);
	CPPUNIT_TEST_SUITE_END();
	
public:
	test_sha1() : CPPUNIT_NS::TestFixture() {}
	virtual ~test_sha1() throw() {}
	void setUp() {}
	void tearDown() {}
	
protected:
	void test_normal();
	void test_hmac();
};

/* ------------------------------------------------------------------------- */
//  registration
/* ------------------------------------------------------------------------- */
CPPUNIT_TEST_SUITE_REGISTRATION(test_sha1);

/* ------------------------------------------------------------------------- */
/*
 *  test_sha1::test_normal
 *
 *  The test code is derived from the following URL:
 *  http://www.faqs.org/rfcs/rfc3174.html.
 */
/* ------------------------------------------------------------------------- */
void test_sha1::test_normal() {
	clx::sha1 sh;
	
	std::string s1 = "abc";
	std::string a1 = "a9993e364706816aba3e25717850c26c9cd0d89d";
	CPPUNIT_ASSERT(sh.encode(s1).to_string() == a1);
	
	std::string s2 = "abcdbcdecdefdefgefghfghighijhijkijkljklmklmnlmnomnopnopq";
	std::string a2 = "84983e441c3bd26ebaae4aa1f95129e5e54670f1";
	CPPUNIT_ASSERT(sh.encode(s2).to_string() == a2);
	
	std::string s3 = "a";
	std::string a3 = "34aa973cd4c4daa4f61eeb2bdbad27316534016f";
	sh.reset();
	for (int i = 0; i < 1000000; i++) sh.update(s3.c_str(), s3.size());
	sh.finish();
	CPPUNIT_ASSERT(sh.to_string() == a3);
	
	std::string s4 = "0123456701234567012345670123456701234567012345670123456701234567";
	std::string a4 = "dea356a2cddd90c7a7ecedc5ebb563934f460452";
	sh.reset();
	for (int i = 0; i < 10; i++) sh.update(s4.c_str(), s4.size());
	sh.finish();
	CPPUNIT_ASSERT(sh.to_string() == a4);
}

/* ------------------------------------------------------------------------- */
/*
 *  test_sha1::test_hmac
 *
 *  The test code is derived from the following URL:
 *  http://www.ipa.go.jp/security/rfc/RFC2202JA.html
 */
/* ------------------------------------------------------------------------- */
void test_sha1::test_hmac() {
	char key1[21];
	char dat1[9] = "Hi There";
	for (size_t i = 0; i < sizeof(key1) - 1; ++i) key1[i] = 0x0b;
	std::string s1 = clx::hmac<clx::sha1>(key1, sizeof(key1) - 1, dat1, sizeof(dat1) - 1).to_string();
	std::string a1 = "b617318655057264e28bc0b6fb378c8ef146be00";
	CPPUNIT_ASSERT(s1 == a1);
	
	char key2[5] = "Jefe";
	char dat2[29] = "what do ya want for nothing?";
	std::string s2 = clx::hmac<clx::sha1>(key2, sizeof(key2) - 1, dat2, sizeof(dat2) - 1).to_string();
	std::string a2 = "effcdf6ae5eb2fa2d27416d5f184df9c259a7c79";
	CPPUNIT_ASSERT(s2 == a2);
	
	char key3[21];
	char dat3[51];
	for (size_t i = 0; i < sizeof(key3) - 1; ++i) key3[i] = 0xaa;
	for (size_t i = 0; i < sizeof(dat3) - 1; ++i) dat3[i] = 0xdd;
	std::string s3 = clx::hmac<clx::sha1>(key3, sizeof(key3) - 1, dat3, sizeof(dat3) - 1).to_string();
	std::string a3 = "125d7342b9ac11cd91a39af48aa17b4f63f175d3";
	CPPUNIT_ASSERT(s3 == a3);
	
	char key4[26];
	char dat4[51];
	for (size_t i = 0; i < sizeof(key4) - 1; ++i) key4[i] = i + 1;
	for (size_t i = 0; i < sizeof(dat4) - 1; ++i) dat4[i] = 0xcd;
	std::string s4 = clx::hmac<clx::sha1>(key4, sizeof(key4) - 1, dat4, sizeof(dat4) - 1).to_string();
	std::string a4 = "4c9007f4026250c6bc8414f9bf50c86c2d7235da";
	CPPUNIT_ASSERT(s4 == a4);
	
	char key5[21];
	char dat5[21] = "Test With Truncation";
	for (size_t i = 0; i < sizeof(key5) - 1; ++i) key5[i] = 0x0c;
	std::string s5 = clx::hmac<clx::sha1>(key5, sizeof(key5) - 1, dat5, sizeof(dat5) - 1).to_string();
	std::string a5 = "4c1a03424b55e07fe7f27be1d58bb9324a9a5a04";
	CPPUNIT_ASSERT(s5 == a5);
	
	char key6[81];
	char dat6[55] = "Test Using Larger Than Block-Size Key - Hash Key First";
	for (size_t i = 0; i < sizeof(key6) - 1; ++i) key6[i] = 0xaa;
	std::string s6 = clx::hmac<clx::sha1>(key6, sizeof(key6) - 1, dat6, sizeof(dat6) - 1).to_string();
	std::string a6 = "aa4ae5e15272d00e95705637ce8a3b55ed402112";
	CPPUNIT_ASSERT(s6 == a6);
	
	char key7[81];
	char dat7[74] = "Test Using Larger Than Block-Size Key and Larger Than One Block-Size Data";
	for (size_t i = 0; i < sizeof(key7) - 1; ++i) key7[i] = 0xaa;
	std::string s7 = clx::hmac<clx::sha1>(key7, sizeof(key7) - 1, dat7, sizeof(dat7) - 1).to_string();
	std::string a7 = "e8e99d0f45237d786d6bbaa7965c7808bbff1a91";
	CPPUNIT_ASSERT(s7 == a7);
}
