/* ------------------------------------------------------------------------- */
/*
 *  test_utf8.cpp
 *
 *  Copyright (c) 2004 - 2010, clown. All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *    - Redistributions of source code must retain the above copyright
 *      notice, this list of conditions and the following disclaimer.
 *    - Redistributions in binary form must reproduce the above copyright
 *      notice, this list of conditions and the following disclaimer in the
 *      documentation and/or other materials provided with the distribution.
 *    - No names of its contributors may be used to endorse or promote
 *      products derived from this software without specific prior written
 *      permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *  Last-modified: Wed 10 Mar 2010 23:26:00 JST
 */
/* ------------------------------------------------------------------------- */
#include <cppunit/extensions/HelperMacros.h>
#include <string>
#include <vector>
#include <iterator>
#include "clx/utf8.h"
#include "clx/hexdump.h"
#include "clx/lexical_cast.h"

/* ------------------------------------------------------------------------- */
//  test_utf8
/* ------------------------------------------------------------------------- */
class test_utf8 : public CPPUNIT_NS::TestFixture {
	CPPUNIT_TEST_SUITE(test_utf8);
	CPPUNIT_TEST(test_roundtrip);
	CPPUNIT_TEST(test_encode_invalid);
	CPPUNIT_TEST(test_decode_invalid);
	CPPUNIT_TEST_SUITE_END();
	
public:
	test_utf8() : CPPUNIT_NS::TestFixture() {}
	virtual ~test_utf8() throw() {}
	void setUp() {}
	void tearDown() {}
	
protected:
	std::string roundtrip(const std::string& src);
	void test_roundtrip();
	void test_encode_invalid();
	void test_decode_invalid();
};

/* ------------------------------------------------------------------------- */
//  registration
/* ------------------------------------------------------------------------- */
CPPUNIT_TEST_SUITE_REGISTRATION(test_utf8);

/* ------------------------------------------------------------------------- */
//  test_utf8::roundtrip
/* ------------------------------------------------------------------------- */
std::string test_utf8::roundtrip(const std::string& src) {
	std::vector<char32_t> decoded;
	for (std::string::const_iterator pos = src.begin(); pos != src.end();) {
		clx::utf8_t c8 = clx::utf8::get(pos, src.end());
		char32_t c32 = clx::utf8::decode(c8);
		decoded.push_back(c32);
	}
	
	std::string dest;
	for (std::vector<char32_t>::const_iterator pos = decoded.begin(); pos != decoded.end(); ++pos) {
		clx::utf8::encode(*pos, std::inserter(dest, dest.end()));
	}
	
	return dest;
}

/* ------------------------------------------------------------------------- */
/*
 *  test_utf8::test_roundtrip
 *
 *  The test function is UTF-8 -> UTF-32 -> UTF-8 senario.
 */
/* ------------------------------------------------------------------------- */
void test_utf8::test_roundtrip() {
	std::string src;
	std::string dest;
	
	src = "a";
	dest = roundtrip(src);
	CPPUNIT_ASSERT_MESSAGE(clx::hexdump(dest), src == dest);
	
	src = "Hello, world!";
	dest = roundtrip(src);
	CPPUNIT_ASSERT_MESSAGE(clx::hexdump(dest), src == dest);
	
	src = "ハローワールド！";
	dest = roundtrip(src);
	CPPUNIT_ASSERT_MESSAGE(clx::hexdump(dest), src == dest);
	
	src = "Hello, world! 日本語との混合テスト";
	dest = roundtrip(src);
	CPPUNIT_ASSERT_MESSAGE(clx::hexdump(dest), src == dest);
	
	src = "𠮟る．surrogates pair in UTF-16";
	dest = roundtrip(src);
	CPPUNIT_ASSERT_MESSAGE(clx::hexdump(dest), src == dest);
	
	src = "";
	dest = roundtrip(src);
	CPPUNIT_ASSERT_MESSAGE(clx::hexdump(dest), dest.empty());
}

/* ------------------------------------------------------------------------- */
/*
 *  test_utf8::test_encode_invalid
 *
 *  Test the surrogates pair character and the out of range.
 *  Note that the current module does not support the UTF-16 surrogates
 *  pair.
 */
/* ------------------------------------------------------------------------- */
void test_utf8::test_encode_invalid() {
	std::vector<char32_t> invalid;
	invalid.push_back(0x0000d842);
	invalid.push_back(0x0000df9f);
	invalid.push_back(0xd842df9f);
	invalid.push_back(0x11111111);
	
	for (std::vector<char32_t>::const_iterator pos = invalid.begin(); pos != invalid.end(); ++pos) {
		try {
			clx::utf8::encode(*pos);
			CPPUNIT_FAIL(clx::lexical_cast<std::string>(*pos, std::ios::hex) + ": must not be encoded");
		}
		catch (std::runtime_error&) {
			// try the next invalid Unicode character.
		}
		catch (...) {
			CPPUNIT_FAIL("unexpected error was occured");
		}
	}
	
	std::vector<char16_t> surrogates;
	surrogates.push_back(0xd842);
	surrogates.push_back(0xdf9f);
	for (std::vector<char16_t>::const_iterator pos = surrogates.begin(); pos != surrogates.end(); ++pos) {
		try {
			clx::utf8::encode(*pos);
			CPPUNIT_FAIL(clx::lexical_cast<std::string>(*pos, std::ios::hex) + ": must not be encoded");
		}
		catch (std::runtime_error&) {
			// try the next invalid Unicode character.
		}
		catch (...) {
			CPPUNIT_FAIL("unexpected error was occured");
		}
	}
}

/* ------------------------------------------------------------------------- */
//  test_utf8::test_decode_invalid
/* ------------------------------------------------------------------------- */
void test_utf8::test_decode_invalid() {
	// Case 1
	const char s1[] = { 0xc0, 0x80 };
	try {
		const char* pos = s1;
		clx::utf8::get_unicode(pos, pos + sizeof(s1));
		std::string tmp(s1);
		CPPUNIT_FAIL(clx::hexdump(tmp) + ": must be invalid character");
	}
	catch (std::runtime_error&) {
		// try the next invalid UTF-8 character.
	}
	catch (...) {
		CPPUNIT_FAIL("unexpected error was occured");
	}
	
	
	// Case 2
	char s2[] = { 0xc0, 0xaf };
	try {
		const char* pos = s2;
		clx::utf8::get_unicode(pos, pos + sizeof(s2));
		std::string tmp(s2);
		CPPUNIT_FAIL(clx::hexdump(tmp) + ": must be invalid character");
	}
	catch (std::runtime_error&) {
		// try the next invalid UTF-8 character.
	}
	catch (...) {
		CPPUNIT_FAIL("unexpected error was occured");
	}
	
	
	// Case 3
	char s3[] = { 0xe0, 0x80, 0xaf };
	try {
		const char* pos = s3;
		clx::utf8::get_unicode(pos, pos + sizeof(s3));
		std::string tmp(s3);
		CPPUNIT_FAIL(clx::hexdump(tmp) + ": must be invalid character");
	}
	catch (std::runtime_error&) {
		// try the next invalid UTF-8 character.
	}
	catch (...) {
		CPPUNIT_FAIL("unexpected error was occured");
	}
	
	
	// Case 4
	char s4[] = { 0xf0, 0x80, 0x80, 0xaf };
	try {
		const char* pos = s4;
		clx::utf8::get_unicode(pos, pos + sizeof(s4));
		std::string tmp(s4);
		CPPUNIT_FAIL(clx::hexdump(tmp) + ": must be invalid character");
	}
	catch (std::runtime_error&) {
		// try the next invalid UTF-8 character.
	}
	catch (...) {
		CPPUNIT_FAIL("unexpected error was occured");
	}
	
	
	// Case 5
	char s5[] = { 0xf8, 0x80, 0x80, 0x80, 0xaf };
	try {
		const char* pos = s5;
		clx::utf8::get_unicode(pos, pos + sizeof(s5));
		std::string tmp(s5);
		CPPUNIT_FAIL(clx::hexdump(tmp) + ": must be invalid character");
	}
	catch (std::runtime_error&) {
		// try the next invalid UTF-8 character.
	}
	catch (...) {
		CPPUNIT_FAIL("unexpected error was occured");
	}
	
	
	// Case 6
	char s6[] = { 0xfa, 0x80, 0x80, 0x80, 0xaf };
	try {
		const char* pos = s6;
		clx::utf8::get_unicode(pos, pos + sizeof(s6));
		std::string tmp(s6);
		CPPUNIT_FAIL(clx::hexdump(tmp) + ": must be invalid character");
	}
	catch (std::runtime_error&) {
		// try the next invalid UTF-8 character.
	}
	catch (...) {
		CPPUNIT_FAIL("unexpected error was occured");
	}
	
	
	// Case 7
	char s7[] = { 0xfc, 0x80, 0x80, 0x80, 0x80, 0xaf };
	try {
		const char* pos = s7;
		clx::utf8::get_unicode(pos, pos + sizeof(s7));
		std::string tmp(s7);
		CPPUNIT_FAIL(clx::hexdump(tmp) + ": must be invalid character");
	}
	catch (std::runtime_error&) {
		// try the next invalid UTF-8 character.
	}
	catch (...) {
		CPPUNIT_FAIL("unexpected error was occured");
	}
	
	
	// Case 8
	char s8[] = { 0xfd, 0x80, 0x80, 0x80, 0x80, 0xaf };
	try {
		const char* pos = s8;
		clx::utf8::get_unicode(pos, pos + sizeof(s8));
		std::string tmp(s8);
		CPPUNIT_FAIL(clx::hexdump(tmp) + ": must be invalid character");
	}
	catch (std::runtime_error&) {
		// try the next invalid UTF-8 character.
	}
	catch (...) {
		CPPUNIT_FAIL("unexpected error was occured");
	}
	
	
	// Case 9
	char s9[] = { 0x2f, 0xc0, 0xae, 0x2e, 0x2f };
	try {
		const char* pos = s9;
		const char* last = pos + sizeof(s9);
		while (pos != last) clx::utf8::get_unicode(pos, last);
		std::string tmp(s9);
		CPPUNIT_FAIL(clx::hexdump(tmp) + ": must be invalid character");
	}
	catch (std::runtime_error&) {
		// try the next invalid UTF-8 character.
	}
	catch (...) {
		CPPUNIT_FAIL("unexpected error was occured");
	}
}
